/*
   Siag, Scheme In A Grid
   Copyright (C) 1996  Ulric Eriksson <ulric@asogy.stockholm.se>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * calc.h
 */

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#define BUFFER_COLS 1000
#define BUFFER_ROWS 1000

#define LINES 0
#define POINTS 1
#define LINESPOINTS 2
#define IMPULSES 3
#define DOTS 4
#define ERRORBARS 5
#define STEPS 6
#define BOXES 7
#define BOXERRORBARS 8

typedef struct {
	int row, col;
} position;

typedef struct _spread {
	unsigned short text;
	double value;
	short type;
	int format;
} spread;

typedef struct _buffer {
	char name[FILENAME_MAX];	/* identifies the buffer */
	char path[FILENAME_MAX];	/* file name */
	int width[BUFFER_COLS + 1];	/* column width in pixels */
	int height[BUFFER_ROWS + 1];	/* row height in pixels */
	position mark_pos;	/* mark */
	spread **matrix;	/* the cell array */
	int change;		/* if there are pending changes */
	struct _buffer *next;	/* next buffer in buffer ring */
} buffer;

typedef struct _window {
	position point_pos;	/* point */
	buffer *buf;		/* buffer */
	position top;		/* left top of visible part of buffer */
	position blku, blkl;	/* upper left & lower right block corners */
	int begin_y;		/* position on the screen */
	int max_y;		/* height in lines */
	Widget viewport,	/* really a form */
		selectall,	/* the small button */
		colnum,		/* column numbers */
		rownum,		/* row numbers */
		grid,		/* cells */
		hscroll,	/* horizontal scrollbar */
		vscroll;	/* vertical scrollbar */
	struct _window *next;	/* next window in window ring */
} window;

/* from window.c */

extern window *w_list;
extern int pr_scr_flag;		/* if the display needs updating */
/*extern int columns, width;*/
/*extern int lines, cols;*/
extern Display *display;
extern Window root/*, cell_win*/;

extern void llpr(char *);
extern window *find_window_by_widget(Widget);
extern void free_window(window *);
extern window *new_window(buffer *, window *);
extern int remove_window(window *);
extern int split_window(window *);
extern void init_windows(buffer *, int, char **);
extern void exit_windows();
extern void show_cur(window *);
extern void hide_cur(window *);

extern void draw_input(Display *, char *);
extern void draw_vbar(Display *, window *);
extern void draw_hbar(Display *, window *);
extern void draw_scrollbars(Display *, window *);
extern void resize_children(Display *, unsigned int, unsigned int);
extern void draw_status(Display *, char *);
extern void draw_rownums(Display *, window *);
extern void draw_colnums(Display *, window *);
extern void draw_cell(Display *, Window, buffer *, int, int, int, int, window *);
extern void draw_cells(Display *, window *);
extern GC get_gc(Display *, Window, unsigned long, unsigned long, Font);
extern void draw_input(Display *, char *);

extern void get_cell_coords(window *, int, int, int, int, int *, int *);
extern void get_coords_cell(window *, int, int, int *, int *, int, int);

/* from matrix.c */
extern int max_columns;
extern int max_lines;
extern spread **new_matrix();
extern void free_matrix(spread **);
extern int init_pool();
extern short pool_size();
extern short pool_install(char *);
extern int ins_data(spread **, short, double, short, int, int);
extern int ins_format(spread **, int, int, int);
extern short ret_text(spread **, int, int);
extern char *ret_taddr(spread **, int, int);
extern double ret_val(spread **, int, int);
extern short ret_type(spread **, int, int);
extern int ret_format(spread **, int, int);
extern int ret_font(spread **, int, int);
extern int line_last_used(spread **);
extern int col_last_used(spread **, int);
extern void downshift_matrix(buffer *, int);
extern void upshift_matrix(buffer *, int);
extern void rightshift_matrix(buffer *, int);
extern void leftshift_matrix(buffer *, int);

/* from cmds.c */
extern void init_cmds();

/* from buffer.c */
extern buffer *b_list;
extern char *buffer_name(char *);
extern buffer *new_buffer(spread **, char *, char *);
extern buffer *free_buffer(buffer *);
extern buffer *find_buffer_by_name(char *);
extern int cell_width(buffer *, int);
extern int cell_height(buffer *, int);

/* from input.c */
extern void DialogDoneAction(Widget, XEvent *, String *, Cardinal *);
extern void DialogCancelAction(Widget, XEvent *, String *, Cardinal *);
extern void init_input();
extern int ask_for_str_comp(char *, char *, int (*)());
extern int ask_for_str(char *, char *);

/* from main.c */
extern XtAppContext app_context;
extern Widget topLevel, topbox, box, form/*, viewport*/, label1, label2/*, selectall,
	colnum, rownum*/, gridpane/*, grid, hscroll, vscroll*/;
/*extern int loopflag;*/
extern int ok2print;
extern char *version;
extern char *siaghome, *siaghelp;
extern void vscroll_jump (Widget, XtPointer, XtPointer);
extern void vscroll_scroll (Widget, XtPointer, XtPointer);
extern void hscroll_jump (Widget, XtPointer, XtPointer);
extern void hscroll_scroll (Widget, XtPointer, XtPointer);

/* from print.c */
extern void plot(int);
extern void preview();
extern void printer();

/* from railway.c */
extern int errorflag;
extern void init_parser(int, char **);
extern int is_constant(char *);
extern double parse_expr(spread **, char *, int, int);
extern int calc_matrix(spread **);
extern void execute(char *);

/* from fileio.c */
extern int savematrix(char *, buffer *);
extern int loadmatrix(char *, buffer *);

/* from position.c */
extern position move_point(window *, position);
extern position point_position(window *);
extern position move_mark(window *, position);
extern position mark_position(window *);
extern position find_beginning_of_buffer(window *);
extern position find_end_of_buffer(window *);
extern int at_end_of_line(window *, position);
extern int at_beginning_of_line(window *, position);
extern position find_beginning_of_line(window *, position);
extern position find_end_of_line(window *, position);
extern position line_forward(window *, position);
extern position line_backward(window *, position);
extern int at_end_of_buffer(window *, position);
extern position cell_forward(window *, position);
extern position cell_backward(window *, position);

/* from mathwrap.c */
extern void init_mathwrap();

/* from filesel.c */
extern int fsel_input(Widget, char *, char *, char **);
extern void fsel_init(Widget);

/* from fontsel.c */
extern void font_init(Widget);
extern int font_input(int *, int *);

/* from dialogs.c */
extern int listsel(XtAppContext, Widget, char *, char **, int);
extern int alertbox(XtAppContext, Widget, char *, char **, int);

