/* txt2siag.c */

/* Converts text files consisting of colonseparated fields
	into Siag format */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include "types.h"
#include "../xstuff/fonts.h"

static char delim = ':';

static double value;

static int
only_space(char *p)
{
	while (*p && isspace(*p)) p++;
	return (*p == '\0');
}

static int
guess_type(char *p)
{
	char *endp;

	if (!strcmp(p, "")) return EMPTY;

	value = strtod(p, &endp);
	if (only_space(endp)) return CONSTANT;

	return LABEL;
}

static void
print_cell(int r, int c, char *p)
{
	switch (guess_type(p)) {
	case EMPTY:
		break;
	case CONSTANT:
	case UNCONSTANT:
	case EXPRESSION:
		/* C expressions are faster for constants, so I'll use those */
		printf("%d %d %d\t==%g\n",
			r, c, SIZE_10 | HELVETICA, value);
		break;
	default:
		printf("%d %d %d\t\"%s\n",
			r, c, SIZE_10 | HELVETICA, p);
		break;
	}
}

int
main(int argc, char **argv)
{
	char b[1024], c[256], *p, *q;
	int row, col;

	if (argc >= 2) delim = argv[1][0];
	printf("# Siag file generated by txt2siag\n");
	printf(".sw 80\n.sh 20\n.sf %d\n", SIZE_10 | HELVETICA);
	row = 1;
	while (fgets(b, 1020, stdin)) {
		printf("# Line %d\n", row);
		col = 1;
		p = b;
		while (*p) {
			q = c;
			while (*p && *p != delim) {
				*q++ = *p++;
			}
			*q = '\0';
			if (*p) p++;
			print_cell(row, col++, c);
		}
		row++;
	}
	printf("# End of file\n");
	return 0;
}

