/*
   Siag, Scheme In A Grid
   Copyright (C) 1996  Ulric Eriksson <ulric@asogy.stockholm.se>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * fileio.c
 * 960209 New file format
 */

/*

   Brief description of the file format:

   #comment                     A comment; ignored
   .sw width                    Standard column width
   .sh height                   Standard row height
   .sf format                   Standard format code
   .cw col width                        Column width in pixels
   .rh row height                       Column height in pixels
   row col format       #               Empty cell
   row col format       "label          Cell containing a label
   row col format       =expression     Cell containing an expression

   Leading or trailing white space is not allowed
   Empty lines are ignored
   All widths and heights are in pixels
   Format codes are in decimal

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>

#include "calc.h"
#include "types.h"
#include "../xstuff/fonts.h"

extern char *getenv();

#if 0	/* this uses stpcpy and isn't used anymore, so I'll just nuke it */
FILE *fopenp(filename, type)
char *filename, *type;
{
	FILE *fp;
	char *path, *dir, *p;
	char fullpath[FILENAME_MAX];

	if ((fp = fopen(filename, type)) == NULL) {
		/* it failed, so let's search the PATH */
		if (filename[0] && filename[0] != '\\' &&
		    filename[0] != '.' && filename[1] != ':') {
			/* there isn't a path there already */
			if ((path = getenv("PATH")) != NULL) {
				if ((path = strdup(path)) == NULL)
					return NULL;
				/* we did find the PATH (should always be the case */
				for (dir = strtok(path, ",;");
				     dir != NULL;
				     dir = strtok((char *) 0, ",;")) {
					p = stpcpy(fullpath, dir);
					*p++ = '\\';
					strcpy(p, filename);
					if ((fp = fopen(fullpath, type)) != NULL)
						break;	/* gotcha! */
				}
				free(path);
			}
		}
	}
	return fp;
}
#endif

static int col_last_changed(spread **matrix, int row, int sf)
{
	int i;

	for (i = max_columns; i > 1; i--)
		if (ret_type(matrix, row, i) == EMPTY &&
			ret_format(matrix, row, i) == sf)
			break;
	return i;
}

int savematrix(char *fn, buffer * buf)
/* Returns: 0 if successful, otherwise 1 */
{
	int i, j, lastcell, lr;
	FILE *fp;
	spread **matrix = buf->matrix;
	int sw = 80, sh = 20, sf = SIZE_10 | HELVETICA;

	if ((fp = fopen(fn, "w")) == (FILE *) 0)
		return 1;

	/* start by saving standard values */
	fprintf(fp, ".sw %d\n", sw);
	fprintf(fp, ".sh %d\n", sh);
	fprintf(fp, ".sf %d\n", sf);

	lr = line_last_used(matrix);
	fprintf(fp, "# %s\n# %d lines\n#\n", fn, lr);

	for (i = 1; i <= BUFFER_ROWS; i++) {
		int height = cell_height(buf, i);
		if (height != sh)
			fprintf(fp, ".rh %d %d\n", i, height);
	}
	for (i = 1; i <= BUFFER_COLS; i++) {
		int width = cell_width(buf, i);
		if (width != sw)
			fprintf(fp, ".cw %d %d\n", i, width);
	}
	for (i = 1; i <= lr; i++) {
		fprintf(fp, "# Line %d\n", i);
		lastcell = col_last_changed(matrix, i, sf);
		for (j = 1; j <= lastcell; j++) {
			switch (ret_type(matrix, i, j)) {
			case EMPTY:
				if (ret_format(matrix, i, j) != sf)
					fprintf(fp, "%d %d %d\t#\n", i, j, ret_format(matrix, i, j));
				break;
			case LABEL:
				fprintf(fp, "%d %d %d\t\"%s\n", i, j,
					ret_format(matrix, i, j),
					ret_taddr(matrix, i, j));
				break;
			case CONSTANT:
			case UNCONSTANT:
	/* these fall through to expression, I don't want to implement them */
			default:	/* EXPRESSION */
				fprintf(fp, "%d %d %d\t=%s\n", i, j,
					ret_format(matrix, i, j),
					ret_taddr(matrix, i, j));
			}
		}
	}
	fprintf(fp, "# End of file %s\n", fn);
	fclose(fp);
	return 0;
}				/* save */

int loadmatrix(char *fn, buffer * buf)
/* Returns: 0 if successful, otherwise 1 */
{
	int i, j;
	short texti;
	FILE *fp;
	char instring[256], *p, *coords, *contents;
	spread **matrix = buf->matrix;
	int sw = 80, sh = 20, sf = SIZE_10 | HELVETICA, f;

	if ((fp = fopen(fn, "r")) == NULL)
		return 1;

	for (i = line_last_used(matrix); i > 0; i--)
		for (j = col_last_changed(matrix, i, sf); j > 0; j--) {
			ins_data(matrix, 0, (double) 0, EMPTY, i, j);
			ins_format(matrix, i, j, sf);
		}
	for (i = 1; i < BUFFER_ROWS; i++)
		buf->height[i] = sh;
	for (i = 1; i < BUFFER_COLS; i++)
		buf->width[i] = sw;

	while (fgets(instring, 250, fp) != NULL) {
		/* strip off \r\n */
/*
   if ((p = strpbrk(instring, "\r\n")) != NULL) *p = '\0';
 */

		switch (instring[0]) {
		case '\0':
		case '#':
			break;
		case '.':
			if (!strncmp(instring, ".sw", 3)) {
				sscanf(instring, ".sw %d", &sw);
				for (i = 1; i < BUFFER_COLS; i++)
					buf->width[i] = sw;
			} else if (!strncmp(instring, ".sh", 3)) {
				sscanf(instring, ".sh %d", &sh);
				for (i = 1; i < BUFFER_COLS; i++)
					buf->height[i] = sh;
			} else if (!strncmp(instring, ".sf", 3)) {
				sscanf(instring, ".sf %d", &sf);
			} else if (!strncmp(instring, ".cw", 3)) {
				int w;
				sscanf(instring, ".cw %d %d", &i, &w);
				buf->width[i] = w;
			} else if (!strncmp(instring, ".rh", 3)) {
				int h;
				sscanf(instring, ".rh %d %d", &i, &h);
				buf->height[i] = h;
			}
			break;
		default:	/* cell definition */
			if ((coords = strtok(instring, "\t")) == NULL)
				break;
			if ((contents = strtok((char *) 0, "\t\n\r")) == NULL)
				break;
			if ((p = strtok(coords, " ")) == NULL)
				break;
			i = atoi(p);
			if ((p = strtok(NULL, " ")) == NULL)
				break;
			j = atoi(p);
			if ((p = strtok(NULL, " ")) == NULL)
				break;
			f = atoi(p);
			switch (contents[0]) {
			case '=':	/* expression */
				if ((texti = pool_install(contents + 1)) == 0)
					return 1;
				ins_data(matrix, texti, (double) 0, EXPRESSION, i, j);
				ins_format(matrix, i, j, f);
				break;
			case '\"':	/* label */
				if ((texti = pool_install(contents + 1)) == 0)
					return 1;
				ins_data(matrix, texti, (double) 0, LABEL, i, j);
				ins_format(matrix, i, j, f);
				break;
			default:	/* anything else means empty */
				ins_format(matrix, i, j, f);
				break;
			}
			break;
		}
	}

	fclose(fp);
	return 0;
} /* load */
