/*
   Siag, Scheme In A Grid
   Copyright (C) 1996  Ulric Eriksson <ulric@asogy.stockholm.se>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *    position.c
 *
 *      This module contains functions for determining positions within
 *      buffers.
 */

#include <stdio.h>
#include <ctype.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>

/*#include "window.h" */
#include "calc.h"

position move_point(window * w, position p)
{
	return w->point_pos = p;
}

position point_position(window * w)
{
	return w->point_pos;
}

position move_mark(window * w, position p)
{
	return w->buf->mark_pos = p;
}

position mark_position(window * w)
{
	return w->buf->mark_pos;
}

position find_beginning_of_buffer(window * w)
{
	position p;
	p.row = p.col = 1;
	return p;
}

position find_end_of_buffer(window * w)
{
	position p;
	p.row = line_last_used(w->buf->matrix);
	p.col = col_last_used(w->buf->matrix, p.row);
	return p;
}

int at_end_of_line(window * w, position p)
{
	return p.col >= line_last_used(w->buf->matrix);
}

int at_beginning_of_line(window * w, position p)
{
	return p.col <= 1;
}

position find_beginning_of_line(window * w, position p)
{
	p.col = 1;
	return p;
}

position find_end_of_line(window * w, position p)
{
	p.col = col_last_used(w->buf->matrix, p.row);
	return p;
}

position line_forward(window * w, position p)
{
	if (p.row < max_lines)
		p.row++;
	return p;
}

position line_backward(window * w, position p)
{
	if (p.row > 1)
		p.row--;
	return p;
}

int at_end_of_buffer(window * w, position p)
{
	return p.row >= max_lines && at_end_of_line(w, p);
}

position cell_forward(window * w, position p)
{
	if (p.col < max_columns)
		p.col++;
	return p;
}

position cell_backward(window * w, position p)
{
	if (p.col > 1)
		p.col--;
	return p;
}
