#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "fonts.h"
#include "xstuff.h"
#include "button.h"
#include "alertbox.h"

static void draw_alert(Display *display, Window wind,
		int ntext, char **text, 
		int width, int height, int text_height)
{
  int i;

  XClearWindow(display, wind);
  XDrawRectangle(display, wind, black_gc, 0, 0,
		width-1, height-1);
  XDrawLine(display, wind, white_gc, 1, 1,
		width-2, 1);
  XDrawLine(display, wind, white_gc, 1, 1,
		1, height-2);
  XDrawLine(display, wind, white_gc, 4, height-5,
		width-5, height-5);
  XDrawLine(display, wind, white_gc, width-5, 4,
		width-5, height-5);
  XDrawLine(display, wind, dark_gc, 4, 4,
		width-5, 4);
  XDrawLine(display, wind, dark_gc, 4, 4,
		4, height-5);
  XDrawLine(display, wind, dark_gc, 1, height-2,
		width-2, height-2);
  XDrawLine(display, wind, dark_gc, width-2, 1,
		width-2, height-2);
  for (i = 0; i < ntext; i++) {
    XDrawString(display, wind, status_gc, 20, text_height*(i+2),
		text[i], strlen(text[i]));
  }
}


int alertbox(Display *display, Window wind, 
		int ntext, char **text, 
		int nbutton, char **button)
{
  Window root, alertw;
  int x, y, alertx, alerty, i, clicked = -1, loopflag;
  unsigned int width, height, border_width, depth, alertwidth, alertheight,
		text_height;
  int screen = DefaultScreen(display);
  Window buttonw[3];

  XGetGeometry(display, wind, &root, &x, &y, &width, &height,
		&border_width, &depth);
  text_height = font_height(display, STATUS_FONT);
  /* calculate the necessary width & height for the alert window */
  alertwidth = 320;
  alertheight = 150;
  alertx = (width-alertwidth)/2;
  alerty = (height-alertheight)/2;
  alertw = XCreateSimpleWindow(display, wind, alertx, alerty, 
			alertwidth, alertheight, 0, 
			BlackPixel(display, screen),
			light_gray.pixel /* WhitePixel(display, screen) */);
  XMapWindow(display, alertw);
  XSelectInput(display, alertw, eventMask);
  draw_alert(display, alertw, ntext, text, alertwidth, alertheight, text_height);
  for (i = 0; i < nbutton; i++) {
    buttonw[i] = XCreateSimpleWindow(display, alertw, 
			20+100*i, alertheight-40,
			80, 20, 0, 
			BlackPixel(display, screen), light_gray.pixel);
    XMapWindow(display, buttonw[i]);
    XSelectInput(display, buttonw[i], eventMask | ButtonReleaseMask);
    draw_button(display, buttonw[i], menu_gc, button[i]);
  }
  /* draw text */
  loopflag = 1;
  while (loopflag) {
    XEvent rep;
    XNextEvent(display, &rep);
    switch (rep.type) {
    case Expose:
      for (i = 0; i < nbutton; i++)
	if (rep.xexpose.window == buttonw[i]) break;
      if (i < nbutton) {
	draw_button(display, buttonw[i], menu_gc, button[i]);
      } else if (rep.xexpose.window == alertw) {
	draw_alert(display, alertw, ntext, text,
			alertwidth, alertheight, text_height);
      } else expose_event(display, rep.xexpose.window);
      break;
    case ButtonPress:
      for (i = 0; i < nbutton; i++)
	if (rep.xbutton.window == buttonw[i]) {
	  clicked = i;
	  break;
	}
      if (i < nbutton) {
	click_button(display, buttonw[i]);
	loopflag = 0;
      }
      break;
    case ConfigureNotify:
      resize_event(display, rep.xconfigure.window,
		rep.xconfigure.x, rep.xconfigure.y,
		rep.xconfigure.width, rep.xconfigure.height);
      break;
    default:
      break;
    }
  }
  for (i = 0; i < nbutton; i++)
    XDestroyWindow(display, buttonw[i]);
  XDestroyWindow(display, alertw);
  return clicked;
}

